#!/usr/bin/env node
'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var main = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
    var argv, endpoint, expectedSchemaFile, expectedSchema, response, _ref2, data, errors, actualSchema, breakingChanges, table;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            argv = (0, _minimist2.default)(process.argv.slice(2));

            if (!(argv._.length < 2)) {
              _context.next = 4;
              break;
            }

            console.log(usage);
            return _context.abrupt('return');

          case 4:
            endpoint = argv._[0];
            expectedSchemaFile = argv._[1];


            console.log('[*] Checking ' + endpoint + ' against ' + expectedSchemaFile + '...\n');

            expectedSchema = void 0;


            _fs2.default.readFile(expectedSchemaFile, 'utf8', function (err, data) {
              if (err) {
                console.log(err);
                process.exit(1);
              }

              expectedSchema = (0, _graphql.buildSchema)(data);
            });

            _context.next = 11;
            return (0, _nodeFetch2.default)(endpoint, {
              method: 'POST',
              headers: {
                'Content-Type': 'application/json'
              },
              body: JSON.stringify({ query: _introspectionQuery.introspectionQuery })
            });

          case 11:
            response = _context.sent;
            _context.next = 14;
            return response.json();

          case 14:
            _ref2 = _context.sent;
            data = _ref2.data;
            errors = _ref2.errors;

            if (!errors) {
              _context.next = 19;
              break;
            }

            throw new Error(JSON.stringify(errors, null, 2));

          case 19:
            actualSchema = (0, _buildClientSchema.buildClientSchema)(data);
            breakingChanges = (0, _findBreakingChanges.findBreakingChanges)(expectedSchema, actualSchema);


            if (breakingChanges.length === 0) {
              console.log(success('✨  The server appears to implement the schema you provided'));
              process.exit(0);
            }

            console.log('\uD83D\uDCA9  ' + error('Breaking changes were detected\n'));

            table = new _cliTable2.default({
              head: ['Change Type', 'Description']
            });


            breakingChanges.forEach(function (change) {
              table.push([change.type, change.description]);
            });

            console.log(table.toString());

            process.exit(1);

          case 27:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function main() {
    return _ref.apply(this, arguments);
  };
}();

var _introspectionQuery = require('graphql/utilities/introspectionQuery');

var _buildClientSchema = require('graphql/utilities/buildClientSchema');

var _findBreakingChanges = require('graphql/utilities/findBreakingChanges');

var _graphql = require('graphql');

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

var _cliTable = require('cli-table');

var _cliTable2 = _interopRequireDefault(_cliTable);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var usage = '  Usage: graphql-contract-test ENDPOINT_URL <client_schema_file>';
var error = _chalk2.default.bold.red;
var success = _chalk2.default.bold.green;

main().catch(function (e) {
  return console.error(e);
});
